
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v6.009.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "ExtUtils::MakeMaker" => 0,
    "Module::Build" => "0.28"
  },
  "dist_abstract" => "the asynchronous, non-blocking shadowsocks client and server. ",
  "dist_author" => [
    "Li ZHOU <lzh\@cpan.org>"
  ],
  "dist_name" => "Net-Shadowsocks",
  "dist_version" => "0.9.3.2",
  "license" => "artistic_2",
  "module_name" => "Net::Shadowsocks",
  "recursive_test_files" => 1,
  "requires" => {
    "AnyEvent" => "4.45",
    "AnyEvent::Handle" => 0,
    "AnyEvent::Log" => 0,
    "AnyEvent::Socket" => 0,
    "Carp" => 0,
    "Crypt::AuthEnc::GCM" => 0,
    "Crypt::Cipher::AES" => 0,
    "Crypt::Cipher::Camellia" => 0,
    "Crypt::Cipher::RC6" => 0,
    "Crypt::KeyDerivation" => 0,
    "Crypt::Mode::CFB" => 0,
    "Crypt::Mode::CTR" => 0,
    "Crypt::Mode::OFB" => 0,
    "Crypt::PRNG" => 0,
    "Crypt::RC4::XS" => 0,
    "Crypt::Spritz" => "1.02",
    "Digest::MD5" => "2.55",
    "Getopt::Std" => 0,
    "IO::Socket::Socks" => "0.73",
    "JSON" => "2.90",
    "Socket" => "2.021",
    "perl" => "5.006",
    "strict" => 0,
    "warnings" => 0
  },
  "script_files" => [
    "bin/ssclient.pl",
    "bin/ssserver.pl"
  ],
  "test_requires" => {
    "ExtUtils::MakeMaker" => "6.3",
    "File::Spec" => 0,
    "IO::Handle" => 0,
    "IPC::Open3" => 0,
    "Pod::Coverage::TrustPod" => 0,
    "Test::CPAN::Meta" => 0,
    "Test::CheckDeps" => "0.010",
    "Test::More" => "0.94",
    "Test::Pod::Coverage" => 0,
    "blib" => "1.01"
  }
);


my %fallback_build_requires = (
  "ExtUtils::MakeMaker" => "6.3",
  "File::Spec" => 0,
  "IO::Handle" => 0,
  "IPC::Open3" => 0,
  "Module::Build" => "0.28",
  "Pod::Coverage::TrustPod" => 0,
  "Test::CPAN::Meta" => 0,
  "Test::CheckDeps" => "0.010",
  "Test::More" => "0.94",
  "Test::Pod::Coverage" => 0,
  "blib" => "1.01"
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

if ( $^O ne 'MSWin32' ) {
	$module_build_args{requires}{'Crypt::NaCl::Sodium'} = '1.0.4';
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
